@@ -17,10 +17,14 @@
  * under the License.
  */
 
+#pragma once
+
 #include <cstdint> // For fixed-width integer types
 
 namespace fory {
 enum class TypeId : int32_t {
+  // Unknown/polymorphic type marker.
+  UNKNOWN = 0,
   // a boolean value (true or false).
   BOOL = 1,
   // an 8-bit signed integer.
@@ -99,14 +103,53 @@ enum class TypeId : int32_t {
   FLOAT32_ARRAY = 36,
   // one-dimensional float64 array.
   FLOAT64_ARRAY = 37,
-  // an arrow record batch object.
-  ARROW_RECORD_BATCH = 38,
-  // an arrow table object.
-  ARROW_TABLE = 39,
-  // Bound value, typically used as a sentinel value.
-  BOUND = 64
+  // an union type that can hold different types of values.
+  UNION = 38,
+  // a null value with no data.
+  NONE = 39,
+  // Unsigned integer types (native mode only, not supported in xlang mode)
+  // an 8-bit unsigned integer.
+  U8 = 64,
+  // a 16-bit unsigned integer.
+  U16 = 65,
+  // a 32-bit unsigned integer.
+  U32 = 66,
+  // a 64-bit unsigned integer.
+  U64 = 67,
+  // 8-bits character.
+  CHAR = 68,
+  // 16-bits character
+  CHAR16 = 69,
+  // 32-bits character
+  CHAR32 = 70,
+  // Unsigned integer array types (native mode only)
+  // one-dimensional uint16 array.
+  U16_ARRAY = 73,
+  // one-dimensional uint32 array.
+  U32_ARRAY = 74,
+  // one-dimensional uint64 array.
+  U64_ARRAY = 75,
+  // Bound value for range checks (types with id >= BOUND are not internal
+  // types).
+  BOUND = 78
 };
 
+inline bool IsUserType(int32_t type_id) {
+  switch (static_cast<TypeId>(type_id)) {
+  case TypeId::ENUM:
+  case TypeId::NAMED_ENUM:
+  case TypeId::STRUCT:
+  case TypeId::COMPATIBLE_STRUCT:
+  case TypeId::NAMED_STRUCT:
+  case TypeId::NAMED_COMPATIBLE_STRUCT:
+  case TypeId::EXT:
+  case TypeId::NAMED_EXT:
+    return true;
+  default:
+    return false;
+  }
+}
+
 inline bool IsNamespacedType(int32_t type_id) {
   switch (static_cast<TypeId>(type_id)) {
   case TypeId::NAMED_ENUM:
@@ -131,4 +174,26 @@ inline bool IsTypeShareMeta(int32_t type_id) {
     return false;
   }
 }
+
+/// Check if type_id represents an internal (built-in) type.
+/// Internal types are all types except user-defined types (ENUM, STRUCT, EXT).
+/// UNKNOWN is excluded because it's a marker for polymorphic types, not a
+/// concrete type.
+/// Keep as constexpr for compile time evaluation or constant folding.
+inline constexpr bool is_internal_type(uint32_t type_id) {
+  if (type_id == 0 || type_id >= static_cast<uint32_t>(TypeId::BOUND)) {
+    return false;
+  }
+  // Internal types are all types that are NOT user types or UNKNOWN
+  uint32_t tid_low = type_id & 0xff;
+  return tid_low != static_cast<uint32_t>(TypeId::ENUM) &&
+         tid_low != static_cast<uint32_t>(TypeId::NAMED_ENUM) &&
+         tid_low != static_cast<uint32_t>(TypeId::STRUCT) &&
+         tid_low != static_cast<uint32_t>(TypeId::COMPATIBLE_STRUCT) &&
+         tid_low != static_cast<uint32_t>(TypeId::NAMED_STRUCT) &&
+         tid_low != static_cast<uint32_t>(TypeId::NAMED_COMPATIBLE_STRUCT) &&
+         tid_low != static_cast<uint32_t>(TypeId::EXT) &&
+         tid_low != static_cast<uint32_t>(TypeId::NAMED_EXT) &&
+         tid_low != static_cast<uint32_t>(TypeId::UNKNOWN);
+}
 } // namespace fory
