@@ -19,9 +19,12 @@
 
 #pragma once
 
-#include "platform.h"
+#include "macros.h"
+#include <array>
 #include <cstdint>
 #include <string>
+#include <string_view>
+#include <utility>
 
 namespace fory {
 
@@ -86,6 +89,98 @@ inline void utf16SurrogatePairToUtf8(uint16_t high, uint16_t low, char *&utf8) {
   *utf8++ = static_cast<char>((code_point & 0x3F) | 0x80);
 }
 
+// Convert Latin1 encoded bytes to UTF-8 string.
+// Latin1 (ISO-8859-1) maps bytes 0-127 to ASCII and bytes 128-255 to
+// Unicode code points U+0080 to U+00FF.
+inline std::string latin1ToUtf8(const uint8_t *data, size_t length) {
+  if (length == 0) {
+    return std::string();
+  }
+
+  // Fast path: if all bytes are ASCII, direct copy
+  if (isAsciiFallback(reinterpret_cast<const char *>(data), length)) {
+    return std::string(reinterpret_cast<const char *>(data), length);
+  }
+
+  // Calculate exact output size to avoid reallocation
+  // ASCII bytes (< 128) need 1 byte, non-ASCII need 2 bytes in UTF-8
+  size_t utf8_len = 0;
+  for (size_t i = 0; i < length; ++i) {
+    utf8_len += (data[i] < 128) ? 1 : 2;
+  }
+
+  std::string result;
+  result.resize(utf8_len);
+  char *out = &result[0];
+
+  for (size_t i = 0; i < length; ++i) {
+    uint8_t byte = data[i];
+    if (byte < 128) {
+      *out++ = static_cast<char>(byte);
+    } else {
+      // Latin1 byte 128-255 maps to U+0080 to U+00FF
+      // UTF-8 encoding: 110xxxxx 10xxxxxx
+      *out++ = static_cast<char>(0xC0 | (byte >> 6));
+      *out++ = static_cast<char>(0x80 | (byte & 0x3F));
+    }
+  }
+
+  return result;
+}
+
+// Convert UTF-16 code units to UTF-8 string.
+// Handles surrogate pairs for characters outside BMP.
+inline std::string utf16ToUtf8(const uint16_t *data, size_t char_count) {
+  if (char_count == 0) {
+    return std::string();
+  }
+
+  // Calculate exact output size
+  // BMP chars < 0x80: 1 byte, < 0x800: 2 bytes, else: 3 bytes
+  // Surrogate pairs: 4 bytes
+  size_t utf8_len = 0;
+  for (size_t i = 0; i < char_count; ++i) {
+    uint16_t ch = data[i];
+    if (ch >= 0xD800 && ch <= 0xDBFF && i + 1 < char_count) {
+      uint16_t low = data[i + 1];
+      if (low >= 0xDC00 && low <= 0xDFFF) {
+        utf8_len += 4; // surrogate pair
+        ++i;
+        continue;
+      }
+    }
+    if (ch < 0x80) {
+      utf8_len += 1;
+    } else if (ch < 0x800) {
+      utf8_len += 2;
+    } else {
+      utf8_len += 3;
+    }
+  }
+
+  std::string result;
+  result.resize(utf8_len);
+  char *out = &result[0];
+
+  for (size_t i = 0; i < char_count; ++i) {
+    uint16_t ch = data[i];
+
+    // Check for surrogate pair
+    if (ch >= 0xD800 && ch <= 0xDBFF && i + 1 < char_count) {
+      uint16_t low = data[i + 1];
+      if (low >= 0xDC00 && low <= 0xDFFF) {
+        utf16SurrogatePairToUtf8(ch, low, out);
+        ++i;
+        continue;
+      }
+    }
+
+    utf16ToUtf8(ch, out);
+  }
+
+  return result;
+}
+
 static inline bool hasSurrogatePairFallback(const uint16_t *data, size_t size) {
   for (size_t i = 0; i < size; ++i) {
     auto c = data[i];
@@ -95,6 +190,85 @@ static inline bool hasSurrogatePairFallback(const uint16_t *data, size_t size) {
   }
   return false;
 }
+
+namespace detail {
+
+constexpr bool is_upper_ascii(char c) { return c >= 'A' && c <= 'Z'; }
+
+constexpr bool is_lower_ascii(char c) { return c >= 'a' && c <= 'z'; }
+
+constexpr bool is_digit_ascii(char c) { return c >= '0' && c <= '9'; }
+
+constexpr char to_lower_ascii(char c) {
+  return is_upper_ascii(c) ? static_cast<char>(c - 'A' + 'a') : c;
+}
+
+constexpr bool needs_snake_separator(std::string_view name, size_t index) {
+  if (index == 0)
+    return false;
+  char curr = name[index];
+  if (!is_upper_ascii(curr))
+    return false;
+
+  char prev = name[index - 1];
+  if (is_lower_ascii(prev) || is_digit_ascii(prev))
+    return true;
+
+  if (is_upper_ascii(prev) && index + 1 < name.size()) {
+    char next = name[index + 1];
+    if (is_lower_ascii(next))
+      return true;
+  }
+  return false;
+}
+
+constexpr size_t snake_case_length(std::string_view name) {
+  size_t length = 0;
+  for (size_t i = 0; i < name.size(); ++i) {
+    char c = name[i];
+    if (c == '_' || c == '-') {
+      ++length;
+      continue;
+    }
+    if (needs_snake_separator(name, i)) {
+      ++length;
+    }
+    ++length;
+  }
+  return length;
+}
+
+template <size_t MaxLength>
+constexpr std::pair<std::array<char, MaxLength + 1>, size_t>
+to_snake_case(std::string_view name) {
+  std::array<char, MaxLength + 1> buffer{};
+  size_t pos = 0;
+  for (size_t i = 0; i < name.size(); ++i) {
+    char c = name[i];
+    if (c == '_' || c == '-') {
+      buffer[pos++] = '_';
+      continue;
+    }
+    if (needs_snake_separator(name, i)) {
+      buffer[pos++] = '_';
+    }
+    buffer[pos++] = is_upper_ascii(c) ? to_lower_ascii(c) : c;
+  }
+  buffer[pos] = '\0';
+  return {buffer, pos};
+}
+
+} // namespace detail
+
+template <size_t MaxLength>
+constexpr std::pair<std::array<char, MaxLength + 1>, size_t>
+to_snake_case(std::string_view name) {
+  return detail::to_snake_case<MaxLength>(name);
+}
+
+constexpr size_t snake_case_length(std::string_view name) {
+  return detail::snake_case_length(name);
+}
 #if defined(FORY_HAS_IMMINTRIN)
 
 inline bool isAscii(const char *data, size_t length) {
