@@ -31,7 +31,6 @@ import org.apache.fory.meta.DeflaterMetaCompressor;
 import org.apache.fory.meta.MetaCompressor;
 import org.apache.fory.pool.ThreadPoolFory;
 import org.apache.fory.reflect.ReflectionUtils;
-import org.apache.fory.resolver.ClassResolver;
 import org.apache.fory.serializer.JavaSerializer;
 import org.apache.fory.serializer.ObjectStreamSerializer;
 import org.apache.fory.serializer.Serializer;
@@ -69,6 +68,8 @@ public final class ForyBuilder {
   ClassLoader classLoader;
   boolean compressInt = true;
   public LongEncoding longEncoding = LongEncoding.SLI;
+  boolean compressIntArray = false;
+  boolean compressLongArray = false;
   boolean compressString = false;
   Boolean writeNumUtf16BytesForUtf8Encoding;
   CompatibleMode compatibleMode = CompatibleMode.SCHEMA_CONSISTENT;
@@ -83,7 +84,7 @@ public final class ForyBuilder {
   boolean registerGuavaTypes = true;
   boolean scalaOptimizationEnabled = false;
   boolean suppressClassRegistrationWarnings = true;
-  boolean deserializeNonexistentEnumValueAsNull = false;
+  UnknownEnumValueStrategy unknownEnumValueStrategy = UnknownEnumValueStrategy.NOT_ALLOWED;
   boolean serializeEnumByName = false;
   int bufferSizeLimitBytes = 128 * 1024;
   MetaCompressor metaCompressor = new DeflaterMetaCompressor();
@@ -97,9 +98,6 @@ public final class ForyBuilder {
    */
   public ForyBuilder withLanguage(Language language) {
     this.language = language;
-    if (language != Language.JAVA) {
-      codeGenEnabled = false;
-    }
     return this;
   }
 
@@ -137,7 +135,19 @@ public final class ForyBuilder {
   /** ignore Enum Deserialize array out of bounds. */
   public ForyBuilder deserializeNonexistentEnumValueAsNull(
       boolean deserializeNonexistentEnumValueAsNull) {
-    this.deserializeNonexistentEnumValueAsNull = deserializeNonexistentEnumValueAsNull;
+    this.unknownEnumValueStrategy = UnknownEnumValueStrategy.RETURN_NULL;
+    return this;
+  }
+
+  /**
+   * Sets the strategy applied when deserialization encounters an enum value that cannot be
+   * resolved.
+   *
+   * @param action policy to apply for unknown enum values
+   * @return this builder instance for chaining
+   */
+  public ForyBuilder withUnknownEnumValueStrategy(UnknownEnumValueStrategy action) {
+    this.unknownEnumValueStrategy = action;
     return this;
   }
 
@@ -185,6 +195,18 @@ public final class ForyBuilder {
     return this;
   }
 
+  /** Whether compress int arrays when values are small. */
+  public ForyBuilder withIntArrayCompressed(boolean intArrayCompressed) {
+    this.compressIntArray = intArrayCompressed;
+    return this;
+  }
+
+  /** Whether compress long arrays when values are small. */
+  public ForyBuilder withLongArrayCompressed(boolean longArrayCompressed) {
+    this.compressLongArray = longArrayCompressed;
+    return this;
+  }
+
   /** Whether compress string for small size. */
   public ForyBuilder withStringCompressed(boolean stringCompressed) {
     this.compressString = stringCompressed;
@@ -242,6 +264,12 @@ public final class ForyBuilder {
    * class won't evolve.
    */
   public ForyBuilder withClassVersionCheck(boolean checkClassVersion) {
+    if (language == Language.XLANG
+        && compatibleMode == CompatibleMode.SCHEMA_CONSISTENT
+        && !checkClassVersion) {
+      throw new IllegalArgumentException(
+          "XLANG Schema consistent mode must enable class version check");
+    }
     this.checkClassVersion = checkClassVersion;
     return this;
   }
@@ -269,8 +297,9 @@ public final class ForyBuilder {
    * attack if the classes `constructor`/`equals`/`hashCode` method contain malicious code. Do not
    * disable class registration if you can't ensure your environment are *indeed secure*. We are not
    * responsible for security risks if you disable this option. If you disable this option, you can
-   * configure {@link org.apache.fory.resolver.ClassChecker} by {@link
-   * ClassResolver#setClassChecker} to control which classes are allowed being serialized.
+   * configure {@link org.apache.fory.resolver.TypeChecker} by {@link
+   * org.apache.fory.resolver.TypeResolver#setTypeChecker} to control which classes are allowed
+   * being serialized.
    */
   public ForyBuilder requireClassRegistration(boolean requireClassRegistration) {
     this.requireClassRegistration = requireClassRegistration;
@@ -393,6 +422,7 @@ public final class ForyBuilder {
       stringRefIgnored = true;
       longEncoding = LongEncoding.PVL;
       compressInt = true;
+      compressString = true;
     }
     if (ENABLE_CLASS_REGISTRATION_FORCIBLY) {
       if (!requireClassRegistration) {
@@ -447,8 +477,8 @@ public final class ForyBuilder {
       LOG.warn(
           "Class registration isn't forced, unknown classes can be deserialized. "
               + "If the environment isn't secure, please enable class registration by "
-              + "`ForyBuilder#requireClassRegistration(true)` or configure ClassChecker by "
-              + "`ClassResolver#setClassChecker`");
+              + "`ForyBuilder#requireClassRegistration(true)` or configure TypeChecker by "
+              + "`TypeResolver#setTypeChecker`");
     }
     if (GraalvmSupport.IN_GRAALVM_NATIVE_IMAGE && asyncCompilationEnabled) {
       LOG.info("Use sync compilation for graalvm native image since it doesn't support JIT.");
