@@ -27,7 +27,6 @@ import static org.apache.fory.meta.Encoders.TYPE_NAME_DECODER;
 import static org.apache.fory.meta.Encoders.encodePackage;
 import static org.apache.fory.meta.Encoders.encodeTypeName;
 import static org.apache.fory.serializer.CodegenSerializer.loadCodegenSerializer;
-import static org.apache.fory.serializer.CodegenSerializer.loadCompatibleCodegenSerializer;
 import static org.apache.fory.serializer.CodegenSerializer.supportCodegenForJavaSerialization;
 import static org.apache.fory.type.TypeUtils.OBJECT_TYPE;
 import static org.apache.fory.type.TypeUtils.getRawType;
@@ -107,9 +106,9 @@ import org.apache.fory.reflect.TypeRef;
 import org.apache.fory.serializer.ArraySerializers;
 import org.apache.fory.serializer.BufferSerializers;
 import org.apache.fory.serializer.CodegenSerializer.LazyInitBeanSerializer;
-import org.apache.fory.serializer.CompatibleSerializer;
 import org.apache.fory.serializer.EnumSerializer;
 import org.apache.fory.serializer.ExternalizableSerializer;
+import org.apache.fory.serializer.FinalFieldReplaceResolveSerializer;
 import org.apache.fory.serializer.ForyCopyableSerializer;
 import org.apache.fory.serializer.JavaSerializer;
 import org.apache.fory.serializer.JdkProxySerializer;
@@ -159,16 +158,52 @@ import org.apache.fory.util.record.RecordUtils;
 /**
  * Class registry for types of serializing objects, responsible for reading/writing types, setting
  * up relations between serializer and types.
+ *
+ * <h2>Class ID Space</h2>
+ *
+ * <p>Fory separates user class IDs from internal class IDs to provide a clean and intuitive API:
+ *
+ * <ul>
+ *   <li><b>User ID space</b>: IDs specified via {@link #register(Class, int)} start from 0. Valid
+ *       range is [0, {@value Short#MAX_VALUE} - {@value #USER_ID_BASE} - 1] (i.e., [0, 32510]).
+ *   <li><b>Internal ID space</b>: Reserved for Fory's built-in types (primitives, common
+ *       collections, etc.). These IDs are in the range [0, {@value #USER_ID_BASE} - 1] and are
+ *       completely hidden from users.
+ * </ul>
+ *
+ * <p>When users register a class with ID N, Fory internally stores it as (N + {@value
+ * #USER_ID_BASE}). This transformation is transparent to users - they only work with their own
+ * 0-based ID space.
+ *
+ * <h2>Registration Methods</h2>
+ *
+ * <ul>
+ *   <li>{@link #register(Class)} - Auto-assigns the next available user ID
+ *   <li>{@link #register(Class, int)} - Registers with a user-specified ID (0-based)
+ *   <li>{@link #register(Class, String, String)} - Registers with namespace and type name
+ * </ul>
+ *
+ * @see #register(Class)
+ * @see #register(Class, int)
  */
 @SuppressWarnings({"rawtypes", "unchecked"})
 public class ClassResolver extends TypeResolver {
   private static final Logger LOG = LoggerFactory.getLogger(ClassResolver.class);
 
-  // preserve 0 as flag for class id not set in ClassInfo`
+  /** Flag value indicating no class ID has been assigned. */
   public static final short NO_CLASS_ID = TypeResolver.NO_CLASS_ID;
+
   public static final short LAMBDA_STUB_ID = 1;
   public static final short JDK_PROXY_STUB_ID = 2;
   public static final short REPLACE_STUB_ID = 3;
+
+  /**
+   * Base offset for user-registered class IDs. User IDs are internally stored as (userId + {@value
+   * #USER_ID_BASE}). The first {@value #USER_ID_BASE} IDs (0 to {@value #USER_ID_BASE} - 1) are
+   * reserved for Fory's internal types.
+   */
+  public static final short USER_ID_BASE = 256;
+
   // Note: following pre-defined class id should be continuous, since they may be used based range.
   public static final short PRIMITIVE_VOID_CLASS_ID = (short) (REPLACE_STUB_ID + 1);
   public static final short PRIMITIVE_BOOLEAN_CLASS_ID = (short) (PRIMITIVE_VOID_CLASS_ID + 1);
@@ -229,43 +264,43 @@ public class ClassResolver extends TypeResolver {
   @Override
   public void initialize() {
     extRegistry.objectGenericType = buildGenericType(OBJECT_TYPE);
-    register(LambdaSerializer.ReplaceStub.class, LAMBDA_STUB_ID);
-    register(JdkProxySerializer.ReplaceStub.class, JDK_PROXY_STUB_ID);
-    register(ReplaceResolveSerializer.ReplaceStub.class, REPLACE_STUB_ID);
-    register(void.class, PRIMITIVE_VOID_CLASS_ID);
-    register(boolean.class, PRIMITIVE_BOOLEAN_CLASS_ID);
-    register(byte.class, PRIMITIVE_BYTE_CLASS_ID);
-    register(char.class, PRIMITIVE_CHAR_CLASS_ID);
-    register(short.class, PRIMITIVE_SHORT_CLASS_ID);
-    register(int.class, PRIMITIVE_INT_CLASS_ID);
-    register(float.class, PRIMITIVE_FLOAT_CLASS_ID);
-    register(long.class, PRIMITIVE_LONG_CLASS_ID);
-    register(double.class, PRIMITIVE_DOUBLE_CLASS_ID);
-    register(Void.class, VOID_CLASS_ID);
-    register(Boolean.class, BOOLEAN_CLASS_ID);
-    register(Byte.class, BYTE_CLASS_ID);
-    register(Character.class, CHAR_CLASS_ID);
-    register(Short.class, SHORT_CLASS_ID);
-    register(Integer.class, INTEGER_CLASS_ID);
-    register(Float.class, FLOAT_CLASS_ID);
-    register(Long.class, LONG_CLASS_ID);
-    register(Double.class, DOUBLE_CLASS_ID);
-    register(String.class, STRING_CLASS_ID);
-    register(boolean[].class, PRIMITIVE_BOOLEAN_ARRAY_CLASS_ID);
-    register(byte[].class, PRIMITIVE_BYTE_ARRAY_CLASS_ID);
-    register(char[].class, PRIMITIVE_CHAR_ARRAY_CLASS_ID);
-    register(short[].class, PRIMITIVE_SHORT_ARRAY_CLASS_ID);
-    register(int[].class, PRIMITIVE_INT_ARRAY_CLASS_ID);
-    register(float[].class, PRIMITIVE_FLOAT_ARRAY_CLASS_ID);
-    register(long[].class, PRIMITIVE_LONG_ARRAY_CLASS_ID);
-    register(double[].class, PRIMITIVE_DOUBLE_ARRAY_CLASS_ID);
-    register(String[].class, STRING_ARRAY_CLASS_ID);
-    register(Object[].class, OBJECT_ARRAY_CLASS_ID);
-    register(ArrayList.class, ARRAYLIST_CLASS_ID);
-    register(HashMap.class, HASHMAP_CLASS_ID);
-    register(HashSet.class, HASHSET_CLASS_ID);
-    register(Class.class, CLASS_CLASS_ID);
-    register(Object.class, EMPTY_OBJECT_ID);
+    registerInternal(LambdaSerializer.ReplaceStub.class, LAMBDA_STUB_ID);
+    registerInternal(JdkProxySerializer.ReplaceStub.class, JDK_PROXY_STUB_ID);
+    registerInternal(ReplaceResolveSerializer.ReplaceStub.class, REPLACE_STUB_ID);
+    registerInternal(void.class, PRIMITIVE_VOID_CLASS_ID);
+    registerInternal(boolean.class, PRIMITIVE_BOOLEAN_CLASS_ID);
+    registerInternal(byte.class, PRIMITIVE_BYTE_CLASS_ID);
+    registerInternal(char.class, PRIMITIVE_CHAR_CLASS_ID);
+    registerInternal(short.class, PRIMITIVE_SHORT_CLASS_ID);
+    registerInternal(int.class, PRIMITIVE_INT_CLASS_ID);
+    registerInternal(float.class, PRIMITIVE_FLOAT_CLASS_ID);
+    registerInternal(long.class, PRIMITIVE_LONG_CLASS_ID);
+    registerInternal(double.class, PRIMITIVE_DOUBLE_CLASS_ID);
+    registerInternal(Void.class, VOID_CLASS_ID);
+    registerInternal(Boolean.class, BOOLEAN_CLASS_ID);
+    registerInternal(Byte.class, BYTE_CLASS_ID);
+    registerInternal(Character.class, CHAR_CLASS_ID);
+    registerInternal(Short.class, SHORT_CLASS_ID);
+    registerInternal(Integer.class, INTEGER_CLASS_ID);
+    registerInternal(Float.class, FLOAT_CLASS_ID);
+    registerInternal(Long.class, LONG_CLASS_ID);
+    registerInternal(Double.class, DOUBLE_CLASS_ID);
+    registerInternal(String.class, STRING_CLASS_ID);
+    registerInternal(boolean[].class, PRIMITIVE_BOOLEAN_ARRAY_CLASS_ID);
+    registerInternal(byte[].class, PRIMITIVE_BYTE_ARRAY_CLASS_ID);
+    registerInternal(char[].class, PRIMITIVE_CHAR_ARRAY_CLASS_ID);
+    registerInternal(short[].class, PRIMITIVE_SHORT_ARRAY_CLASS_ID);
+    registerInternal(int[].class, PRIMITIVE_INT_ARRAY_CLASS_ID);
+    registerInternal(float[].class, PRIMITIVE_FLOAT_ARRAY_CLASS_ID);
+    registerInternal(long[].class, PRIMITIVE_LONG_ARRAY_CLASS_ID);
+    registerInternal(double[].class, PRIMITIVE_DOUBLE_ARRAY_CLASS_ID);
+    registerInternal(String[].class, STRING_ARRAY_CLASS_ID);
+    registerInternal(Object[].class, OBJECT_ARRAY_CLASS_ID);
+    registerInternal(ArrayList.class, ARRAYLIST_CLASS_ID);
+    registerInternal(HashMap.class, HASHMAP_CLASS_ID);
+    registerInternal(HashSet.class, HASHSET_CLASS_ID);
+    registerInternal(Class.class, CLASS_CLASS_ID);
+    registerInternal(Object.class, EMPTY_OBJECT_ID);
     registerCommonUsedClasses();
     registerDefaultClasses();
     addDefaultSerializers();
@@ -320,7 +355,7 @@ public class ClassResolver extends TypeResolver {
             Objects.requireNonNull(classInfoMap.get(NonexistentMetaShared.class)).classId;
         Preconditions.checkArgument(classId > 63 && classId < 8192, classId);
       } else {
-        register(NonexistentSkip.class);
+        registerInternal(NonexistentSkip.class);
       }
     }
   }
@@ -330,121 +365,121 @@ public class ClassResolver extends TypeResolver {
   }
 
   private void addDefaultSerializer(Class type, Serializer serializer) {
-    registerSerializer(type, serializer);
-    register(type);
+    registerInternalSerializer(type, serializer);
+    registerInternal(type);
   }
 
   /** Register common class ahead to get smaller class id for serialization. */
   private void registerCommonUsedClasses() {
-    register(LinkedList.class, TreeSet.class);
-    register(LinkedHashMap.class, TreeMap.class);
-    register(Date.class, Timestamp.class, LocalDateTime.class, Instant.class);
-    register(BigInteger.class, BigDecimal.class);
-    register(Optional.class, OptionalInt.class);
-    register(Boolean[].class, Byte[].class, Short[].class, Character[].class);
-    register(Integer[].class, Float[].class, Long[].class, Double[].class);
+    registerInternal(LinkedList.class, TreeSet.class);
+    registerInternal(LinkedHashMap.class, TreeMap.class);
+    registerInternal(Date.class, Timestamp.class, LocalDateTime.class, Instant.class);
+    registerInternal(BigInteger.class, BigDecimal.class);
+    registerInternal(Optional.class, OptionalInt.class);
+    registerInternal(Boolean[].class, Byte[].class, Short[].class, Character[].class);
+    registerInternal(Integer[].class, Float[].class, Long[].class, Double[].class);
   }
 
   private void registerDefaultClasses() {
-    register(Platform.HEAP_BYTE_BUFFER_CLASS);
-    register(Platform.DIRECT_BYTE_BUFFER_CLASS);
-    register(Comparator.naturalOrder().getClass());
-    register(Comparator.reverseOrder().getClass());
-    register(ConcurrentHashMap.class);
-    register(ArrayBlockingQueue.class);
-    register(LinkedBlockingQueue.class);
-    register(AtomicBoolean.class);
-    register(AtomicInteger.class);
-    register(AtomicLong.class);
-    register(AtomicReference.class);
-    register(EnumSet.allOf(Language.class).getClass());
-    register(EnumSet.of(Language.JAVA).getClass());
-    register(SerializedLambda.class);
-    register(
+    registerInternal(Platform.HEAP_BYTE_BUFFER_CLASS);
+    registerInternal(Platform.DIRECT_BYTE_BUFFER_CLASS);
+    registerInternal(Comparator.naturalOrder().getClass());
+    registerInternal(Comparator.reverseOrder().getClass());
+    registerInternal(ConcurrentHashMap.class);
+    registerInternal(ArrayBlockingQueue.class);
+    registerInternal(LinkedBlockingQueue.class);
+    registerInternal(AtomicBoolean.class);
+    registerInternal(AtomicInteger.class);
+    registerInternal(AtomicLong.class);
+    registerInternal(AtomicReference.class);
+    registerInternal(EnumSet.allOf(Language.class).getClass());
+    registerInternal(EnumSet.of(Language.JAVA).getClass());
+    registerInternal(SerializedLambda.class);
+    registerInternal(
         Throwable.class,
         StackTraceElement.class,
         StackTraceElement[].class,
         Exception.class,
         RuntimeException.class);
-    register(NullPointerException.class);
-    register(IOException.class);
-    register(IllegalArgumentException.class);
-    register(IllegalStateException.class);
-    register(IndexOutOfBoundsException.class, ArrayIndexOutOfBoundsException.class);
+    registerInternal(NullPointerException.class);
+    registerInternal(IOException.class);
+    registerInternal(IllegalArgumentException.class);
+    registerInternal(IllegalStateException.class);
+    registerInternal(IndexOutOfBoundsException.class, ArrayIndexOutOfBoundsException.class);
   }
 
-  /** register class. */
+  /**
+   * Registers a class with an auto-assigned user ID.
+   *
+   * <p>The ID is automatically assigned starting from 0 in the user ID space. Each call assigns the
+   * next available ID. If the class is already registered, this method does nothing.
+   *
+   * <p>Example:
+   *
+   * <pre>{@code
+   * fory.register(MyClass.class);      // Gets user ID 0
+   * fory.register(AnotherClass.class); // Gets user ID 1
+   * }</pre>
+   *
+   * @param cls the class to register
+   */
+  @Override
   public void register(Class<?> cls) {
     if (!extRegistry.registeredClassIdMap.containsKey(cls)) {
-      while (extRegistry.classIdGenerator < registeredId2ClassInfo.length
-          && registeredId2ClassInfo[extRegistry.classIdGenerator] != null) {
-        extRegistry.classIdGenerator++;
+      while (extRegistry.userIdGenerator + USER_ID_BASE < registeredId2ClassInfo.length
+          && registeredId2ClassInfo[extRegistry.userIdGenerator + USER_ID_BASE] != null) {
+        extRegistry.userIdGenerator++;
       }
-      register(cls, extRegistry.classIdGenerator);
+      register(cls, extRegistry.userIdGenerator);
     }
   }
 
+  /**
+   * Registers a class by its fully qualified name with an auto-assigned user ID.
+   *
+   * @param className the fully qualified class name
+   * @see #register(Class)
+   */
+  @Override
   public void register(String className) {
     register(loadClass(className, false, 0, false));
   }
 
-  public void register(Class<?>... classes) {
-    for (Class<?> cls : classes) {
-      register(cls);
-    }
-  }
-
   /**
-   * This method has been deprecated, please use {@link #register(Class)} instead, and invoke {@link
-   * #ensureSerializersCompiled} after all classes has been registered.
+   * Registers a class by its fully qualified name with a specified user ID.
+   *
+   * @param className the fully qualified class name
+   * @param classId the user ID to assign (0-based, in user ID space)
+   * @see #register(Class, int)
    */
-  @Deprecated
-  public void register(Class<?> cls, boolean createSerializer) {
-    register(cls);
-  }
-
-  /**
-   * Register class with specified id. Currently class id must be `classId >= 0 && classId < 32767`.
-   * In the future this limitation may be relaxed.
-   */
-  public void register(Class<?> cls, int classId) {
-    checkRegisterAllowed();
-    // class id must be less than Integer.MAX_VALUE/2 since we use bit 0 as class id flag.
-    Preconditions.checkArgument(classId >= 0 && classId < Short.MAX_VALUE);
-    short id = (short) classId;
-    checkRegistration(cls, id, cls.getName());
-    extRegistry.registeredClassIdMap.put(cls, id);
-    if (registeredId2ClassInfo.length <= id) {
-      ClassInfo[] tmp = new ClassInfo[(id + 1) * 2];
-      System.arraycopy(registeredId2ClassInfo, 0, tmp, 0, registeredId2ClassInfo.length);
-      registeredId2ClassInfo = tmp;
-    }
-    ClassInfo classInfo = classInfoMap.get(cls);
-    if (classInfo != null) {
-      classInfo.classId = id;
-    } else {
-      classInfo = new ClassInfo(this, cls, null, id, NOT_SUPPORT_XLANG);
-      // make `extRegistry.registeredClassIdMap` and `classInfoMap` share same classInfo
-      // instances.
-      classInfoMap.put(cls, classInfo);
-    }
-    // serializer will be set lazily in `addSerializer` method if it's null.
-    registeredId2ClassInfo[id] = classInfo;
-    extRegistry.registeredClasses.put(cls.getName(), cls);
-    extRegistry.classIdGenerator++;
-  }
-
+  @Override
   public void register(String className, int classId) {
     register(loadClass(className, false, 0, false), classId);
   }
 
   /**
-   * This method has been deprecated, please use {@link #register(Class, int)} instead, and invoke
-   * {@link #ensureSerializersCompiled} after all classes has been registered.
+   * Registers a class with a user-specified ID.
+   *
+   * <p>The ID is in the user ID space, starting from 0. Fory internally transforms this to an
+   * internal ID by adding {@link #USER_ID_BASE}. This separation ensures user IDs never conflict
+   * with Fory's internal type IDs.
+   *
+   * <p>Valid user ID range: [0, 32510] (i.e., [0, Short.MAX_VALUE - USER_ID_BASE - 1])
+   *
+   * <p>Example:
+   *
+   * <pre>{@code
+   * fory.register(MyClass.class, 0);      // User ID 0 -> Internal ID 256
+   * fory.register(AnotherClass.class, 1); // User ID 1 -> Internal ID 257
+   * }</pre>
+   *
+   * @param cls the class to register
+   * @param id the user ID to assign (0-based)
+   * @throws IllegalArgumentException if the ID is out of valid range or already in use
    */
-  @Deprecated
-  public void register(Class<?> cls, int id, boolean createSerializer) {
-    register(cls, id);
+  @Override
+  public void register(Class<?> cls, int id) {
+    registerImpl(cls, id + USER_ID_BASE);
   }
 
   /**
@@ -478,6 +513,84 @@ public class ClassResolver extends TypeResolver {
     compositeNameBytes2ClassInfo.put(
         new TypeNameBytes(nsBytes.hashCode, nameBytes.hashCode), classInfo);
     extRegistry.registeredClasses.put(fullname, cls);
+    GraalvmSupport.registerClass(cls, fory.getConfig().getConfigHash());
+  }
+
+  /**
+   * Registers multiple classes for internal use with auto-assigned internal IDs.
+   *
+   * <p><b>Internal API</b>: This method is for Fory's internal use only. Users should use {@link
+   * #register(Class)} instead.
+   *
+   * @param classes the classes to register
+   */
+  public void registerInternal(Class<?>... classes) {
+    for (Class<?> cls : classes) {
+      registerInternal(cls);
+    }
+  }
+
+  /**
+   * Registers a class for internal use with an auto-assigned internal ID.
+   *
+   * <p><b>Internal API</b>: This method is for Fory's internal use only. Users should use {@link
+   * #register(Class)} instead. Internal IDs are in the range [0, {@link #USER_ID_BASE} - 1].
+   *
+   * @param cls the class to register
+   */
+  public void registerInternal(Class<?> cls) {
+    if (!extRegistry.registeredClassIdMap.containsKey(cls)) {
+      while (extRegistry.classIdGenerator < registeredId2ClassInfo.length
+          && registeredId2ClassInfo[extRegistry.classIdGenerator] != null) {
+        extRegistry.classIdGenerator++;
+      }
+      registerInternal(cls, extRegistry.classIdGenerator);
+    }
+  }
+
+  /**
+   * Registers a class for internal use with a specified internal ID.
+   *
+   * <p><b>Internal API</b>: This method is for Fory's internal use only. Users should use {@link
+   * #register(Class, int)} instead.
+   *
+   * <p>Internal IDs are reserved for Fory's built-in types and must be in the range [0, {@link
+   * #USER_ID_BASE} - 1] (i.e., [0, 255]). User IDs start from {@link #USER_ID_BASE} and above.
+   *
+   * @param cls the class to register
+   * @param classId the internal ID, must be in range [0, {@link #USER_ID_BASE} - 1]
+   * @throws IllegalArgumentException if the ID is out of range or already in use
+   */
+  public void registerInternal(Class<?> cls, int classId) {
+    Preconditions.checkArgument(classId >= 0 && classId < USER_ID_BASE);
+    registerImpl(cls, classId);
+  }
+
+  private void registerImpl(Class<?> cls, int classId) {
+    checkRegisterAllowed();
+    // class id must be less than Integer.MAX_VALUE/2 since we use bit 0 as class id flag.
+    Preconditions.checkArgument(classId >= 0 && classId < Short.MAX_VALUE);
+    short id = (short) classId;
+    checkRegistration(cls, id, cls.getName());
+    extRegistry.registeredClassIdMap.put(cls, id);
+    if (registeredId2ClassInfo.length <= id) {
+      ClassInfo[] tmp = new ClassInfo[(id + 1) * 2];
+      System.arraycopy(registeredId2ClassInfo, 0, tmp, 0, registeredId2ClassInfo.length);
+      registeredId2ClassInfo = tmp;
+    }
+    ClassInfo classInfo = classInfoMap.get(cls);
+    if (classInfo != null) {
+      classInfo.classId = id;
+    } else {
+      classInfo = new ClassInfo(this, cls, null, id, NOT_SUPPORT_XLANG);
+      // make `extRegistry.registeredClassIdMap` and `classInfoMap` share same classInfo
+      // instances.
+      classInfoMap.put(cls, classInfo);
+    }
+    // serializer will be set lazily in `addSerializer` method if it's null.
+    registeredId2ClassInfo[id] = classInfo;
+    extRegistry.registeredClasses.put(cls.getName(), cls);
+    GraalvmSupport.registerClass(cls, fory.getConfig().getConfigHash());
   }
 
   private void checkRegistration(Class<?> cls, short classId, String name) {
@@ -525,6 +638,7 @@ public class ClassResolver extends TypeResolver {
 
   public Tuple2<String, String> getRegisteredNameTuple(Class<?> cls) {
     String name = extRegistry.registeredClasses.inverse().get(cls);
+    Preconditions.checkNotNull(name);
     int index = name.lastIndexOf(".");
     if (index != -1) {
       return Tuple2.of(name.substring(0, index), name.substring(index + 1));
@@ -670,23 +784,38 @@ public class ClassResolver extends TypeResolver {
    * @param <T> type of class
    */
   public <T> void registerSerializer(Class<T> type, Class<? extends Serializer> serializerClass) {
-    checkRegisterAllowed();
     registerSerializer(type, Serializers.newSerializer(fory, type, serializerClass));
   }
 
+  @Override
+  public void registerSerializer(Class<?> type, Serializer<?> serializer) {
+    checkRegisterAllowed();
+    if (!serializer.getClass().getPackage().getName().startsWith("org.apache.fory")) {
+      SerializationUtils.validate(type, serializer.getClass());
+    }
+    if (!extRegistry.registeredClassIdMap.containsKey(type) && !fory.isCrossLanguage()) {
+      register(type);
+    }
+    registerSerializerImpl(type, serializer);
+  }
+
   /**
    * If a serializer exists before, it will be replaced by new serializer.
    *
    * @param type class needed to be serialized/deserialized
    * @param serializer serializer for object of {@code type}
    */
-  public void registerSerializer(Class<?> type, Serializer<?> serializer) {
+  public void registerInternalSerializer(Class<?> type, Serializer<?> serializer) {
+    registerSerializerImpl(type, serializer);
+  }
+
+  private void registerSerializerImpl(Class<?> type, Serializer<?> serializer) {
     checkRegisterAllowed();
     if (!serializer.getClass().getPackage().getName().startsWith("org.apache.fory")) {
       SerializationUtils.validate(type, serializer.getClass());
     }
     if (!extRegistry.registeredClassIdMap.containsKey(type) && !fory.isCrossLanguage()) {
-      register(type);
+      registerInternal(type);
     }
     addSerializer(type, serializer);
     ClassInfo classInfo = classInfoMap.get(type);
@@ -789,8 +918,8 @@ public class ClassResolver extends TypeResolver {
     }
   }
 
-  /** Ass serializer for specified class. */
-  private void addSerializer(Class<?> type, Serializer<?> serializer) {
+  /** Add serializer for specified class. */
+  public void addSerializer(Class<?> type, Serializer<?> serializer) {
     Preconditions.checkNotNull(serializer);
     // 1. Try to get ClassInfo from `registeredId2ClassInfo` and
     // `classInfoMap` or create a new `ClassInfo`.
@@ -801,7 +930,8 @@ public class ClassResolver extends TypeResolver {
     if (registered) {
       classInfo = registeredId2ClassInfo[classId];
     } else {
-      if (serializer instanceof ReplaceResolveSerializer) {
+      if (serializer instanceof ReplaceResolveSerializer
+          && !(serializer instanceof FinalFieldReplaceResolveSerializer)) {
         classId = REPLACE_STUB_ID;
       } else {
         classId = NO_CLASS_ID;
@@ -1029,16 +1159,13 @@ public class ClassResolver extends TypeResolver {
                           callback);
               return sc;
             case COMPATIBLE:
-              // If share class meta, compatible serializer won't be necessary, class
-              // definition will be sent to peer to create serializer for deserialization.
+              // Always use ObjectSerializer for compatible mode.
+              // Class definition will be sent to peer to create serializer for deserialization.
               sc =
                   fory.getJITContext()
                       .registerSerializerJITCallback(
-                          () -> shareMeta ? ObjectSerializer.class : CompatibleSerializer.class,
-                          () ->
-                              shareMeta
-                                  ? loadCodegenSerializer(fory, cls)
-                                  : loadCompatibleCodegenSerializer(fory, cls),
+                          () -> ObjectSerializer.class,
+                          () -> loadCodegenSerializer(fory, cls),
                           callback);
               return sc;
             default:
@@ -1053,15 +1180,8 @@ public class ClassResolver extends TypeResolver {
       if (codegen) {
         LOG.info("Object of type {} can't be serialized by jit", cls);
       }
-      switch (fory.getCompatibleMode()) {
-        case SCHEMA_CONSISTENT:
-          return ObjectSerializer.class;
-        case COMPATIBLE:
-          return shareMeta ? ObjectSerializer.class : CompatibleSerializer.class;
-        default:
-          throw new UnsupportedOperationException(
-              String.format("Unsupported mode %s", fory.getCompatibleMode()));
-      }
+      // Always use ObjectSerializer for both modes
+      return ObjectSerializer.class;
     }
   }
 
@@ -1089,18 +1209,6 @@ public class ClassResolver extends TypeResolver {
         };
   }
 
-  public FieldResolver getFieldResolver(Class<?> cls) {
-    // can't use computeIfAbsent, since there may be recursive multiple
-    // `getFieldResolver` thus multiple updates, which cause concurrent
-    // modification exception.
-    FieldResolver fieldResolver = extRegistry.fieldResolverMap.get(cls);
-    if (fieldResolver == null) {
-      fieldResolver = FieldResolver.of(fory, cls);
-      extRegistry.fieldResolverMap.put(cls, fieldResolver);
-    }
-    return fieldResolver;
-  }
-
   @Override
   public List<Descriptor> getFieldDescriptors(Class<?> clz, boolean searchParent) {
     SortedMap<Member, Descriptor> allDescriptors = getAllDescriptorsMap(clz, searchParent);
@@ -1800,14 +1908,26 @@ public class ClassResolver extends TypeResolver {
           fory, JdkProxySerializer.SUBT_PROXY.getClass(), JdkProxySerializer.class);
       classInfoMap.forEach(
           (cls, classInfo) -> {
+            GraalvmSupport.registerClass(cls, fory.getConfig().getConfigHash());
             if (classInfo.serializer == null) {
               if (isSerializable(classInfo.cls)) {
                 createSerializer0(cls);
               }
               if (cls.isArray()) {
+                // Also create serializer for the component type
                 createSerializer0(TypeUtils.getArrayComponent(cls));
               }
             }
+            // Always ensure array class serializers and their component type serializers
+            // are registered in GraalVM registry, since ObjectArraySerializer needs
+            // the component type serializer at construction time
+            if (cls.isArray() && GraalvmSupport.isGraalBuildtime()) {
+              // First ensure component type serializer is registered
+              Class<?> componentType = TypeUtils.getArrayComponent(cls);
+              createSerializer0(componentType);
+              // Then register the array serializer
+              createSerializer0(cls);
+            }
           });
       if (GraalvmSupport.isGraalBuildtime()) {
         classInfoCache = NIL_CLASS_INFO;
