@@ -26,9 +26,7 @@ import com.google.common.collect.HashBiMap;
 import java.lang.reflect.Field;
 import java.lang.reflect.Member;
 import java.lang.reflect.Type;
-import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -36,7 +34,6 @@ import java.util.Map;
 import java.util.Set;
 import java.util.SortedMap;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import java.util.function.Function;
 import org.apache.fory.Fory;
 import org.apache.fory.annotation.Internal;
@@ -120,26 +117,59 @@ public abstract class TypeResolver {
     }
   }
 
+  /**
+   * Registers a class with an auto-assigned user ID.
+   *
+   * @param type the class to register
+   */
   public abstract void register(Class<?> type);
 
+  /**
+   * Registers a class with a user-specified ID. Valid ID range is [0, 32510].
+   *
+   * @param type the class to register
+   * @param id the user ID to assign (0-based)
+   */
   public abstract void register(Class<?> type, int id);
 
+  /**
+   * Registers a class with a namespace and type name for cross-language serialization.
+   *
+   * @param type the class to register
+   * @param namespace the namespace (can be empty if type name has no conflict)
+   * @param typeName the type name
+   */
   public abstract void register(Class<?> type, String namespace, String typeName);
 
+  /** Registers a class by name with an auto-assigned user ID. */
   public void register(String className) {
     register(loadClass(className));
   }
 
+  /** Registers a class by name with a user-specified ID. */
   public void register(String className, int classId) {
     register(loadClass(className), classId);
   }
 
+  /** Registers a class by name with a namespace and type name. */
   public void register(String className, String namespace, String typeName) {
     register(loadClass(className), namespace, typeName);
   }
 
+  /**
+   * Registers a custom serializer for a type.
+   *
+   * @param type the class to register
+   * @param serializer the serializer instance to use
+   */
   public abstract void registerSerializer(Class<?> type, Serializer<?> serializer);
 
+  /**
+   * Registers a custom serializer class for a type.
+   *
+   * @param type the class to register
+   * @param serializerClass the serializer class (will be instantiated by Fory)
+   */
   public abstract <T> void registerSerializer(
       Class<T> type, Class<? extends Serializer> serializerClass);
 
@@ -581,34 +611,18 @@ public abstract class TypeResolver {
     extRegistry.typeChecker = typeChecker;
   }
 
-  private static final ConcurrentMap<Integer, GraalvmClassRegistry> GRAALVM_REGISTRY =
-      new ConcurrentHashMap<>();
-
   // CHECKSTYLE.OFF:MethodName
   public static void _addGraalvmClassRegistry(int foryConfigHash, ClassResolver classResolver) {
     // CHECKSTYLE.ON:MethodName
     if (GraalvmSupport.isGraalBuildtime()) {
-      GraalvmClassRegistry registry =
-          GRAALVM_REGISTRY.computeIfAbsent(foryConfigHash, k -> new GraalvmClassRegistry());
+      GraalvmSupport.GraalvmClassRegistry registry =
+          GraalvmSupport.getClassRegistry(foryConfigHash);
       registry.resolvers.add(classResolver);
     }
   }
 
-  static class GraalvmClassRegistry {
-    final List<ClassResolver> resolvers;
-    final Map<Class<?>, Class<? extends Serializer>> serializerClassMap;
-    final Map<Long, Class<? extends Serializer>> deserializerClassMap;
-
-    private GraalvmClassRegistry() {
-      resolvers = Collections.synchronizedList(new ArrayList<>());
-      serializerClassMap = new ConcurrentHashMap<>();
-      deserializerClassMap = new ConcurrentHashMap<>();
-    }
-  }
-
-  final GraalvmClassRegistry getGraalvmClassRegistry() {
-    return GRAALVM_REGISTRY.computeIfAbsent(
-        fory.getConfig().getConfigHash(), k -> new GraalvmClassRegistry());
+  final GraalvmSupport.GraalvmClassRegistry getGraalvmClassRegistry() {
+    return GraalvmSupport.getClassRegistry(fory.getConfig().getConfigHash());
   }
 
   final Class<? extends Serializer> getGraalvmSerializerClass(Serializer serializer) {
@@ -619,13 +633,13 @@ public abstract class TypeResolver {
   }
 
   final Class<? extends Serializer> getSerializerClassFromGraalvmRegistry(Class<?> cls) {
-    GraalvmClassRegistry registry = getGraalvmClassRegistry();
-    List<ClassResolver> classResolvers = registry.resolvers;
-    if (classResolvers.isEmpty()) {
+    GraalvmSupport.GraalvmClassRegistry registry = getGraalvmClassRegistry();
+    List<TypeResolver> resolvers = registry.resolvers;
+    if (resolvers.isEmpty()) {
       return null;
     }
-    for (ClassResolver classResolver : classResolvers) {
-      if (classResolver != this) {
+    for (TypeResolver resolver : resolvers) {
+      if (resolver != this) {
         ClassInfo classInfo = getClassInfo(cls, false);
         if (classInfo != null && classInfo.serializer != null) {
           return classInfo.serializer.getClass();
@@ -648,9 +662,9 @@ public abstract class TypeResolver {
 
   private Class<? extends Serializer> getMetaSharedDeserializerClassFromGraalvmRegistry(
       Class<?> cls, ClassDef classDef) {
-    GraalvmClassRegistry registry = getGraalvmClassRegistry();
-    List<ClassResolver> classResolvers = registry.resolvers;
-    if (classResolvers.isEmpty()) {
+    GraalvmSupport.GraalvmClassRegistry registry = getGraalvmClassRegistry();
+    List<TypeResolver> resolvers = registry.resolvers;
+    if (resolvers.isEmpty()) {
       return null;
     }
     Class<? extends Serializer> deserializerClass =
@@ -683,6 +697,7 @@ public abstract class TypeResolver {
     // Here we set it to 1 because `NO_CLASS_ID` is 0 to avoid calculating it again in
     // `register(Class<?> cls)`.
     short classIdGenerator = 1;
+    short userIdGenerator = 0;
     SerializerFactory serializerFactory;
     final IdentityMap<Class<?>, Short> registeredClassIdMap =
         new IdentityMap<>(estimatedNumRegistered);
@@ -693,7 +708,6 @@ public abstract class TypeResolver {
     // avoid potential recursive call for seq codec generation.
     // ex. A->field1: B, B.field1: A
     final Set<Class<?>> getClassCtx = new HashSet<>();
-    final Map<Class<?>, FieldResolver> fieldResolverMap = new HashMap<>();
     final LongMap<Tuple2<ClassDef, ClassInfo>> classIdToDef = new LongMap<>();
     final Map<Class<?>, ClassDef> currentLayerClassDef = new HashMap<>();
     // Tuple2<Class, Class>: Tuple2<From Class, To Class>
