@@ -56,6 +56,7 @@ import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.LongStream;
@@ -474,23 +475,25 @@ public class CollectionSerializersTest extends ForyTestBase {
             .withRefTracking(true)
             .requireClassRegistration(false)
             .build();
-    // TODO(chaokunyang) add optimized serializers for blocking queue.
     {
       ArrayBlockingQueue<Integer> queue = new ArrayBlockingQueue<>(10);
       queue.add(1);
       queue.add(2);
       queue.add(3);
-      assertEquals(new ArrayList<>(serDe(fory, queue)), new ArrayList<>(queue));
+      ArrayBlockingQueue<Integer> deserialized = serDe(fory, queue);
+      assertEquals(new ArrayList<>(deserialized), new ArrayList<>(queue));
+      // Verify capacity is preserved
+      assertEquals(deserialized.remainingCapacity() + deserialized.size(), 10);
     }
     {
-      // If reference tracking is off, deserialization will throw
-      // `java.lang.IllegalMonitorStateException`
-      // when using fory `ObjectStreamSerializer`, maybe some internal state are shared.
       LinkedBlockingQueue<Integer> queue = new LinkedBlockingQueue<>(10);
       queue.add(1);
       queue.add(2);
       queue.add(3);
-      assertEquals(new ArrayList<>(serDe(fory, queue)), new ArrayList<>(queue));
+      LinkedBlockingQueue<Integer> deserialized = serDe(fory, queue);
+      assertEquals(new ArrayList<>(deserialized), new ArrayList<>(queue));
+      // Verify capacity is preserved
+      assertEquals(deserialized.remainingCapacity() + deserialized.size(), 10);
     }
   }
 
@@ -503,17 +506,18 @@ public class CollectionSerializersTest extends ForyTestBase {
       queue.add(3);
       ArrayBlockingQueue<Integer> copy = fory.copy(queue);
       Assert.assertEquals(Arrays.toString(copy.toArray()), "[1, 2, 3]");
+      // Verify capacity is preserved
+      Assert.assertEquals(copy.remainingCapacity() + copy.size(), 10);
     }
     {
-      // If reference tracking is off, deserialization will throw
-      // `java.lang.IllegalMonitorStateException`
-      // when using fory `ObjectStreamSerializer`, maybe some internal state are shared.
       LinkedBlockingQueue<Integer> queue = new LinkedBlockingQueue<>(10);
       queue.add(1);
       queue.add(2);
       queue.add(3);
       LinkedBlockingQueue<Integer> copy = fory.copy(queue);
       Assert.assertEquals(Arrays.toString(copy.toArray()), "[1, 2, 3]");
+      // Verify capacity is preserved
+      Assert.assertEquals(copy.remainingCapacity() + copy.size(), 10);
     }
   }
 
@@ -965,4 +969,21 @@ public class CollectionSerializersTest extends ForyTestBase {
             .build();
     serDeCheck(fory, obj);
   }
+
+  @Test
+  public void testCopyOnWriteArraySet() {
+    final CopyOnWriteArraySet<String> set =
+        new CopyOnWriteArraySet<>(Arrays.asList("Value1", "Value2"));
+    Assert.assertEquals(set, serDe(getJavaFory(), set));
+    Assert.assertEquals(
+        getJavaFory().getClassResolver().getSerializerClass(CopyOnWriteArraySet.class),
+        CollectionSerializers.CopyOnWriteArraySetSerializer.class);
+  }
+
+  @Test(dataProvider = "foryCopyConfig")
+  public void testCopyOnWriteArraySet(Fory fory) {
+    final CopyOnWriteArraySet<Object> set =
+        new CopyOnWriteArraySet<>(Arrays.asList("a", "b", Cyclic.create(true)));
+    copyCheck(fory, set);
+  }
 }
