@@ -254,7 +254,7 @@ class TypeResolver:
 
     def _initialize_common(self):
         register = functools.partial(self._register_type, internal=True)
-        register(None, type_id=TypeId.NA, serializer=NoneSerializer)
+        register(None, type_id=TypeId.UNKNOWN, serializer=NoneSerializer)
         register(bool, type_id=TypeId.BOOL, serializer=BooleanSerializer)
         register(int8, type_id=TypeId.INT8, serializer=ByteSerializer)
         register(int16, type_id=TypeId.INT16, serializer=Int16Serializer)
@@ -301,21 +301,6 @@ class TypeResolver:
         register(list, type_id=TypeId.LIST, serializer=ListSerializer)
         register(set, type_id=TypeId.SET, serializer=SetSerializer)
         register(dict, type_id=TypeId.MAP, serializer=MapSerializer)
-        try:
-            import pyarrow as pa
-            from pyfory.format.serializer import (
-                ArrowRecordBatchSerializer,
-                ArrowTableSerializer,
-            )
-
-            register(
-                pa.RecordBatch,
-                type_id=TypeId.ARROW_RECORD_BATCH,
-                serializer=ArrowRecordBatchSerializer,
-            )
-            register(pa.Table, type_id=TypeId.ARROW_TABLE, serializer=ArrowTableSerializer)
-        except Exception:
-            pass
 
     def register_type(
         self,
