@@ -22,7 +22,36 @@ use crate::meta::{
     FIELD_NAME_ENCODER, NAMESPACE_DECODER, TYPE_NAME_DECODER,
 };
 use crate::resolver::type_resolver::{TypeInfo, TypeResolver};
-use crate::types::{TypeId, PRIMITIVE_TYPES};
+use crate::types::{
+    TypeId, COMPATIBLE_STRUCT, ENUM, EXT, NAMED_COMPATIBLE_STRUCT, NAMED_ENUM, NAMED_EXT,
+    NAMED_STRUCT, PRIMITIVE_TYPES, STRUCT, UNKNOWN,
+};
+
+/// Normalizes a type ID for comparison purposes in cross-language schema evolution.
+/// This treats all struct variants (STRUCT, COMPATIBLE_STRUCT, NAMED_STRUCT,
+/// NAMED_COMPATIBLE_STRUCT) and UNKNOWN as equivalent to STRUCT.
+/// UNKNOWN (0) is used for polymorphic types (interfaces) in cross-language serialization.
+/// Similarly for ENUM and EXT variants.
+fn normalize_type_id_for_eq(type_id: u32) -> u32 {
+    let low = type_id & 0xff;
+    match low {
+        // All struct variants and UNKNOWN normalize to STRUCT
+        _ if low == STRUCT
+            || low == COMPATIBLE_STRUCT
+            || low == NAMED_STRUCT
+            || low == NAMED_COMPATIBLE_STRUCT
+            || low == UNKNOWN =>
+        {
+            STRUCT
+        }
+        // All enum variants normalize to ENUM
+        _ if low == ENUM || low == NAMED_ENUM => ENUM,
+        // All ext variants normalize to EXT
+        _ if low == EXT || low == NAMED_EXT => EXT,
+        // Everything else stays the same
+        _ => type_id,
+    }
+}
 use std::clone::Clone;
 use std::cmp::min;
 use std::collections::HashMap;
@@ -281,12 +310,17 @@ pub fn sort_fields(
 
 impl PartialEq for FieldType {
     fn eq(&self, other: &Self) -> bool {
-        self.type_id == other.type_id && self.generics == other.generics
+        // Normalize type IDs for comparison to handle cross-language schema evolution.
+        // This allows UNKNOWN (0) polymorphic types to match STRUCT (15) in Rust.
+        normalize_type_id_for_eq(self.type_id) == normalize_type_id_for_eq(other.type_id)
+            && self.generics == other.generics
     }
 }
 
 #[derive(Debug)]
-pub struct TypeMetaLayer {
+pub struct TypeMeta {
+    // assigned valid value and used, only during deserializing
+    hash: i64,
     type_id: u32,
     namespace: Rc<MetaString>,
     type_name: Rc<MetaString>,
@@ -294,15 +328,16 @@ pub struct TypeMetaLayer {
     field_infos: Vec<FieldInfo>,
 }
 
-impl TypeMetaLayer {
+impl TypeMeta {
     pub fn new(
         type_id: u32,
         namespace: MetaString,
         type_name: MetaString,
         register_by_name: bool,
         field_infos: Vec<FieldInfo>,
-    ) -> TypeMetaLayer {
-        TypeMetaLayer {
+    ) -> TypeMeta {
+        TypeMeta {
+            hash: 0,
             type_id,
             namespace: Rc::from(namespace),
             type_name: Rc::from(type_name),
@@ -311,8 +346,35 @@ impl TypeMetaLayer {
         }
     }
 
-    pub fn empty() -> TypeMetaLayer {
-        TypeMetaLayer {
+    #[inline(always)]
+    pub fn get_field_infos(&self) -> &Vec<FieldInfo> {
+        &self.field_infos
+    }
+
+    #[inline(always)]
+    pub fn get_type_id(&self) -> u32 {
+        self.type_id
+    }
+
+    #[inline(always)]
+    pub fn get_hash(&self) -> i64 {
+        self.hash
+    }
+
+    #[inline(always)]
+    pub fn get_type_name(&self) -> Rc<MetaString> {
+        self.type_name.clone()
+    }
+
+    #[inline(always)]
+    pub fn get_namespace(&self) -> Rc<MetaString> {
+        self.namespace.clone()
+    }
+
+    #[inline(always)]
+    pub fn empty() -> TypeMeta {
+        TypeMeta {
+            hash: 0,
             type_id: 0,
             namespace: Rc::from(MetaString::get_empty().clone()),
             type_name: Rc::from(MetaString::get_empty().clone()),
@@ -321,26 +383,11 @@ impl TypeMetaLayer {
         }
     }
 
-    pub fn get_type_id(&self) -> u32 {
-        self.type_id
-    }
-
-    pub fn get_type_name(&self) -> Rc<MetaString> {
-        self.type_name.clone()
-    }
-
-    pub fn get_namespace(&self) -> Rc<MetaString> {
-        self.namespace.clone()
-    }
-
-    pub fn get_field_infos(&self) -> &Vec<FieldInfo> {
-        &self.field_infos
-    }
-
     /// Creates a deep clone with new Rc instances.
     /// This is safe for concurrent use from multiple threads.
-    pub fn deep_clone(&self) -> TypeMetaLayer {
-        TypeMetaLayer {
+    pub fn deep_clone(&self) -> TypeMeta {
+        TypeMeta {
+            hash: self.hash,
             type_id: self.type_id,
             namespace: Rc::new((*self.namespace).clone()),
             type_name: Rc::new((*self.type_name).clone()),
@@ -349,6 +396,16 @@ impl TypeMetaLayer {
         }
     }
 
+    pub(crate) fn from_fields(
+        type_id: u32,
+        namespace: MetaString,
+        type_name: MetaString,
+        register_by_name: bool,
+        field_infos: Vec<FieldInfo>,
+    ) -> TypeMeta {
+        TypeMeta::new(type_id, namespace, type_name, register_by_name, field_infos)
+    }
+
     fn write_name(writer: &mut Writer, name: &MetaString, encodings: &[Encoding]) {
         let encoding_idx = encodings.iter().position(|x| *x == name.encoding).unwrap() as u8;
         let bytes = name.bytes.as_slice();
@@ -395,9 +452,9 @@ impl TypeMetaLayer {
         Self::read_name(reader, &TYPE_NAME_DECODER, TYPE_NAME_ENCODINGS)
     }
 
-    fn to_bytes(&self) -> Result<Vec<u8>, Error> {
+    fn to_meta_bytes(&self) -> Result<Vec<u8>, Error> {
         let mut buffer = vec![];
-        // layer_bytes:| meta_header | fields meta |
+        // meta_bytes:| meta_header | fields meta |
         let mut writer = Writer::from_buffer(&mut buffer);
         let num_fields = self.field_infos.len();
         let _internal_id = self.type_id & 0xff;
@@ -531,10 +588,10 @@ impl TypeMetaLayer {
         sorted_field_infos
     }
 
-    fn from_bytes(
+    fn from_meta_bytes(
         reader: &mut Reader,
         type_resolver: &TypeResolver,
-    ) -> Result<TypeMetaLayer, Error> {
+    ) -> Result<TypeMeta, Error> {
         let meta_header = reader.read_u8()?;
         let register_by_name = (meta_header & REGISTER_BY_NAME_FLAG) != 0;
         let mut num_fields = meta_header as usize & SMALL_NUM_FIELDS_THRESHOLD;
@@ -571,7 +628,7 @@ impl TypeMetaLayer {
             Self::assign_field_ids(&type_info_current, &mut sorted_field_infos);
         }
         // if no type found, keep all fields id as -1 to be skipped.
-        Ok(TypeMetaLayer::new(
+        Ok(TypeMeta::new(
             type_id,
             namespace,
             type_name,
@@ -591,9 +648,9 @@ impl TypeMetaLayer {
         for field in field_infos.iter_mut() {
             match field_info_map.get(&field.field_name.clone()) {
                 Some(local_field_info) => {
-                    if field.field_type.type_id != local_field_info.field_type.type_id
-                        || field.field_type.generics != local_field_info.field_type.generics
-                    {
+                    // Use FieldType comparison which normalizes type IDs for cross-language
+                    // schema evolution (e.g., UNKNOWN=0 matches STRUCT variants)
+                    if field.field_type != local_field_info.field_type {
                         field.field_id = -1;
                     } else {
                         field.field_id = local_field_info.field_id;
@@ -605,70 +662,6 @@ impl TypeMetaLayer {
             }
         }
     }
-}
-
-#[derive(Debug)]
-pub struct TypeMeta {
-    // assigned valid value and used, only during deserializing
-    hash: i64,
-    layer: TypeMetaLayer,
-}
-
-impl TypeMeta {
-    #[inline(always)]
-    pub fn get_field_infos(&self) -> &Vec<FieldInfo> {
-        self.layer.get_field_infos()
-    }
-
-    #[inline(always)]
-    pub fn get_type_id(&self) -> u32 {
-        self.layer.get_type_id()
-    }
-
-    #[inline(always)]
-    pub fn get_hash(&self) -> i64 {
-        self.hash
-    }
-
-    #[inline(always)]
-    pub fn get_type_name(&self) -> Rc<MetaString> {
-        self.layer.get_type_name()
-    }
-
-    #[inline(always)]
-    pub fn get_namespace(&self) -> Rc<MetaString> {
-        self.layer.get_namespace()
-    }
-
-    #[inline(always)]
-    pub fn empty() -> TypeMeta {
-        TypeMeta {
-            hash: 0,
-            layer: TypeMetaLayer::empty(),
-        }
-    }
-
-    /// Creates a deep clone with new Rc instances.
-    /// This is safe for concurrent use from multiple threads.
-    pub fn deep_clone(&self) -> TypeMeta {
-        TypeMeta {
-            hash: self.hash,
-            layer: self.layer.deep_clone(),
-        }
-    }
-
-    pub(crate) fn from_fields(
-        type_id: u32,
-        namespace: MetaString,
-        type_name: MetaString,
-        register_by_name: bool,
-        field_infos: Vec<FieldInfo>,
-    ) -> TypeMeta {
-        TypeMeta {
-            hash: 0,
-            layer: TypeMetaLayer::new(type_id, namespace, type_name, register_by_name, field_infos),
-        }
-    }
 
     #[allow(dead_code)]
     pub(crate) fn from_bytes(
@@ -688,11 +681,9 @@ impl TypeMeta {
 
         // let current_meta_size = 0;
         // while current_meta_size < meta_size {}
-        let layer = TypeMetaLayer::from_bytes(reader, type_resolver)?;
-        Ok(TypeMeta {
-            layer,
-            hash: meta_hash,
-        })
+        let mut meta = Self::from_meta_bytes(reader, type_resolver)?;
+        meta.hash = meta_hash;
+        Ok(meta)
     }
 
     pub(crate) fn from_bytes_with_header(
@@ -712,11 +703,9 @@ impl TypeMeta {
 
         // let current_meta_size = 0;
         // while current_meta_size < meta_size {}
-        let layer = TypeMetaLayer::from_bytes(reader, type_resolver)?;
-        Ok(TypeMeta {
-            layer,
-            hash: meta_hash,
-        })
+        let mut meta = Self::from_meta_bytes(reader, type_resolver)?;
+        meta.hash = meta_hash;
+        Ok(meta)
     }
 
     #[inline(always)]
@@ -746,17 +735,14 @@ impl TypeMeta {
     }
 
     pub(crate) fn to_bytes(&self) -> Result<Vec<u8>, Error> {
-        // | global_binary_header | layers_bytes |
+        // | global_binary_header | meta_bytes |
         let mut buffer = vec![];
         let mut result = Writer::from_buffer(&mut buffer);
-        let mut layers_buffer = vec![];
-        let mut layers_writer = Writer::from_buffer(&mut layers_buffer);
-        // for layer in self.layers.iter() {
-        //     layers_writer.bytes(layer.to_bytes()?.as_slice());
-        // }
-        layers_writer.write_bytes(self.layer.to_bytes()?.as_slice());
+        let mut meta_buffer = vec![];
+        let mut meta_writer = Writer::from_buffer(&mut meta_buffer);
+        meta_writer.write_bytes(self.to_meta_bytes()?.as_slice());
         // global_binary_header:| hash:50bits | is_compressed:1bit | write_fields_meta:1bit | meta_size:12bits |
-        let meta_size = layers_writer.len() as i64;
+        let meta_size = meta_writer.len() as i64;
         let mut header: i64 = min(META_SIZE_MASK, meta_size);
         let write_meta_fields_flag = !self.get_field_infos().is_empty();
         if write_meta_fields_flag {
@@ -766,13 +752,13 @@ impl TypeMeta {
         if is_compressed {
             header |= COMPRESS_META_FLAG;
         }
-        let meta_hash = murmurhash3_x64_128(layers_writer.dump().as_slice(), 47).0 as i64;
+        let meta_hash = murmurhash3_x64_128(meta_writer.dump().as_slice(), 47).0 as i64;
         header |= (meta_hash << (64 - NUM_HASH_BITS)).abs();
         result.write_i64(header);
         if meta_size >= META_SIZE_MASK {
             result.write_varuint32((meta_size - META_SIZE_MASK) as u32);
         }
-        result.write_bytes(layers_buffer.as_slice());
+        result.write_bytes(meta_buffer.as_slice());
         Ok(buffer)
     }
 }
