@@ -43,6 +43,7 @@ type ReadFn =
 
 type WriteDataFn = fn(&dyn Any, &mut WriteContext, has_generics: bool) -> Result<(), Error>;
 type ReadDataFn = fn(&mut ReadContext) -> Result<Box<dyn Any>, Error>;
+type ReadCompatibleFn = fn(&mut ReadContext, Rc<TypeInfo>) -> Result<Box<dyn Any>, Error>;
 type ToSerializerFn = fn(Box<dyn Any>) -> Result<Box<dyn Serializer>, Error>;
 type BuildTypeInfosFn = fn(&TypeResolver) -> Result<Vec<(std::any::TypeId, TypeInfo)>, Error>;
 const EMPTY_STRING: String = String::new();
@@ -53,6 +54,7 @@ pub struct Harness {
     read_fn: ReadFn,
     write_data_fn: WriteDataFn,
     read_data_fn: ReadDataFn,
+    read_compatible_fn: Option<ReadCompatibleFn>,
     to_serializer: ToSerializerFn,
     build_type_infos: BuildTypeInfosFn,
 }
@@ -63,6 +65,7 @@ impl Harness {
         read_fn: ReadFn,
         write_data_fn: WriteDataFn,
         read_data_fn: ReadDataFn,
+        read_compatible_fn: Option<ReadCompatibleFn>,
         to_serializer: ToSerializerFn,
         build_type_infos: BuildTypeInfosFn,
     ) -> Harness {
@@ -71,6 +74,7 @@ impl Harness {
             read_fn,
             write_data_fn,
             read_data_fn,
+            read_compatible_fn,
             to_serializer,
             build_type_infos,
         }
@@ -82,6 +86,7 @@ impl Harness {
             stub_read_fn,
             stub_write_data_fn,
             stub_read_data_fn,
+            None,
             stub_to_serializer_fn,
             stub_build_type_infos,
         )
@@ -107,10 +112,33 @@ impl Harness {
         self.read_data_fn
     }
 
+    #[inline(always)]
+    pub fn get_read_compatible_fn(&self) -> Option<ReadCompatibleFn> {
+        self.read_compatible_fn
+    }
+
     #[inline(always)]
     pub fn get_to_serializer(&self) -> ToSerializerFn {
         self.to_serializer
     }
+
+    /// Reads polymorphic data using the appropriate function based on mode.
+    /// In compatible mode, uses read_compatible_fn if available to handle schema
+    /// evolution. Otherwise, uses read_data_fn for direct deserialization.
+    #[inline(always)]
+    pub fn read_polymorphic_data(
+        &self,
+        context: &mut ReadContext,
+        typeinfo: &Rc<TypeInfo>,
+    ) -> Result<Box<dyn Any>, Error> {
+        if context.is_compatible() {
+            if let Some(read_compatible_fn) = self.read_compatible_fn {
+                // Only clone when actually needed for compatible mode
+                return read_compatible_fn(context, typeinfo.clone());
+            }
+        }
+        (self.read_data_fn)(context)
+    }
 }
 
 #[derive(Clone, Debug)]
@@ -234,6 +262,7 @@ impl TypeInfo {
                 stub_read_fn,
                 stub_write_data_fn,
                 stub_read_data_fn,
+                None,
                 stub_to_serializer_fn,
                 stub_build_type_infos,
             )
@@ -700,11 +729,19 @@ impl TypeResolver {
             build_struct_type_infos::<T>(type_resolver)
         }
 
+        fn read_compatible<T2: 'static + StructSerializer + ForyDefault>(
+            context: &mut ReadContext,
+            type_info: Rc<TypeInfo>,
+        ) -> Result<Box<dyn Any>, Error> {
+            Ok(Box::new(T2::fory_read_compatible(context, type_info)?))
+        }
+
         let harness = Harness::new(
             write::<T>,
             read::<T>,
             write_data::<T>,
             read_data::<T>,
+            Some(read_compatible::<T>),
             to_serializer::<T>,
             build_type_infos::<T>,
         );
@@ -726,10 +763,10 @@ impl TypeResolver {
 
         // Check if type_id conflicts with any already registered type
         // Skip check for:
-        // 1. Internal types (type_id <= TypeId::UNKNOWN) as they can be shared
+        // 1. Internal types (type_id < TypeId::BOUND) as they can be shared
         // 2. Types registered by name (they use shared type IDs like NAMED_STRUCT)
         if !register_by_name
-            && actual_type_id > TypeId::UNKNOWN as u32
+            && actual_type_id >= TypeId::BOUND as u32
             && self.type_info_map_by_id.contains_key(&actual_type_id)
         {
             return Err(Error::type_error(format!(
@@ -891,11 +928,13 @@ impl TypeResolver {
             build_serializer_type_infos(partial_info, std::any::TypeId::of::<T2>())
         }
 
+        // EXT types don't support fory_read_compatible
         let harness = Harness::new(
             write::<T>,
             read::<T>,
             write_data::<T>,
             read_data::<T>,
+            None,
             to_serializer::<T>,
             build_type_infos::<T>,
         );
@@ -917,8 +956,8 @@ impl TypeResolver {
         }
 
         // Check if type_id conflicts with any already registered type
-        // Skip check for internal types (type_id <= TypeId::UNKNOWN) as they can be shared
-        if actual_type_id > TypeId::UNKNOWN as u32
+        // Skip check for internal types (type_id < TypeId::BOUND) as they can be shared
+        if actual_type_id >= TypeId::BOUND as u32
             && self.type_info_map_by_id.contains_key(&actual_type_id)
         {
             return Err(Error::type_error(format!(
