@@ -1073,7 +1073,7 @@ pub trait Serializer: 'static {
     ///
     /// # Type ID Categories
     ///
-    /// - **Primitives**: `I8`, `I16`, `I32`, `I64`, `U8`, `U16`, `U32`, `U64`, `F32`, `F64`, `BOOL`
+    /// - **Primitives**: `BOOL`, `I8`, `I16`, `I32`, `I64`, `U8`, `U16`, `U32`, `U64`, `USIZE`, `U128`, `F32`, `F64`
     /// - **Strings**: `STRING`
     /// - **Collections**: `LIST`, `MAP`, `SET`
     /// - **Structs**: `STRUCT`
@@ -1126,9 +1126,10 @@ pub trait Serializer: 'static {
     where
         Self: Sized,
     {
-        Ok(type_resolver
-            .get_type_info(&std::any::TypeId::of::<Self>())?
-            .get_type_id())
+        match type_resolver.get_type_info(&std::any::TypeId::of::<Self>()) {
+            Ok(info) => Ok(info.get_type_id()),
+            Err(e) => Err(Error::enhance_type_error::<Self>(e)),
+        }
     }
 
     /// **[USER IMPLEMENTATION REQUIRED]** Get the runtime type ID for this instance.
