@@ -17,6 +17,7 @@
 
 pub mod any;
 mod arc;
+mod array;
 mod bool;
 mod box_;
 pub mod collection;
@@ -36,6 +37,8 @@ pub mod skip;
 mod string;
 pub mod struct_;
 pub mod trait_object;
+mod tuple;
+mod unsigned_number;
 pub mod util;
 pub mod weak;
 
