@@ -23,7 +23,8 @@ use crate::serializer::collection::{DECL_ELEMENT_TYPE, HAS_NULL, IS_SAME_TYPE};
 use crate::serializer::util;
 use crate::serializer::Serializer;
 use crate::types;
-use crate::types::{is_user_type, RefFlag};
+use crate::types::RefFlag;
+use crate::util::ENABLE_FORY_DEBUG_OUTPUT;
 use chrono::{NaiveDate, NaiveDateTime};
 use std::rc::Rc;
 use std::time::Duration;
@@ -50,32 +51,95 @@ pub fn skip_any_value(context: &mut ReadContext, read_ref_flag: bool) -> Result<
         if ref_flag == (RefFlag::Null as i8) {
             return Ok(());
         }
+        if ref_flag == (RefFlag::Ref as i8) {
+            // Reference to already-seen object, skip the reference index
+            let _ref_index = context.reader.read_varuint32()?;
+            return Ok(());
+        }
+        // RefValue (0) or NotNullValue (-1) means we need to read the actual object
     }
 
-    // Now read the type ID
-    let mut type_id = context.reader.peek_u8()? as u32;
-    if !is_user_type(type_id) {
-        type_id = context.reader.read_varuint32()?;
-    }
-    let field_type = match type_id {
-        types::LIST | types::SET => FieldType {
-            type_id,
-            nullable: true,
-            generics: vec![UNKNOWN_FIELD_TYPE],
-        },
-        types::MAP => FieldType {
-            type_id,
-            nullable: true,
-            generics: vec![UNKNOWN_FIELD_TYPE, UNKNOWN_FIELD_TYPE],
-        },
-        _ => FieldType {
-            type_id,
-            nullable: true,
-            generics: vec![],
-        },
+    // Read type_id first
+    let type_id = context.reader.read_varuint32()?;
+    let internal_id = type_id & 0xff;
+
+    // For struct-like types, also read meta_index to get type_info
+    // This is critical for polymorphic collections where elements are struct types.
+    let (field_type, type_info_opt) = match internal_id {
+        types::LIST | types::SET => (
+            FieldType {
+                type_id,
+                nullable: true,
+                generics: vec![UNKNOWN_FIELD_TYPE],
+            },
+            None,
+        ),
+        types::MAP => (
+            FieldType {
+                type_id,
+                nullable: true,
+                generics: vec![UNKNOWN_FIELD_TYPE, UNKNOWN_FIELD_TYPE],
+            },
+            None,
+        ),
+        types::COMPATIBLE_STRUCT | types::NAMED_COMPATIBLE_STRUCT => {
+            // For compatible struct types, read meta_index to get type_info
+            let meta_index = context.reader.read_varuint32()? as usize;
+            let type_info = context.get_type_info_by_index(meta_index)?.clone();
+            (
+                FieldType {
+                    type_id,
+                    nullable: true,
+                    generics: vec![],
+                },
+                Some(type_info),
+            )
+        }
+        types::STRUCT | types::NAMED_STRUCT => {
+            // For non-compatible struct types with share_meta enabled, read meta_index
+            if context.is_share_meta() {
+                let meta_index = context.reader.read_varuint32()? as usize;
+                let type_info = context.get_type_info_by_index(meta_index)?.clone();
+                (
+                    FieldType {
+                        type_id,
+                        nullable: true,
+                        generics: vec![],
+                    },
+                    Some(type_info),
+                )
+            } else {
+                // Without share_meta, read namespace and type_name
+                let namespace = context.read_meta_string()?.to_owned();
+                let type_name = context.read_meta_string()?.to_owned();
+                let rc_namespace = Rc::from(namespace);
+                let rc_type_name = Rc::from(type_name);
+                let type_info = context
+                    .get_type_resolver()
+                    .get_type_info_by_meta_string_name(rc_namespace, rc_type_name)
+                    .ok_or_else(|| crate::Error::type_error("Name harness not found"))?;
+                (
+                    FieldType {
+                        type_id,
+                        nullable: true,
+                        generics: vec![],
+                    },
+                    Some(type_info),
+                )
+            }
+        }
+        _ => (
+            FieldType {
+                type_id,
+                nullable: true,
+                generics: vec![],
+            },
+            None,
+        ),
     };
-    // Don't read ref flag again in skip_value since we already handled it
-    skip_value(context, &field_type, false, false, &None)
+    // Don't read ref flag again in skip_value since we already handled it.
+    // Pass type_info so skip_struct doesn't try to read type_id/meta_index again.
+    skip_value(context, &field_type, false, false, &type_info_opt)
 }
 
 fn skip_collection(context: &mut ReadContext, field_type: &FieldType) -> Result<(), Error> {
@@ -240,9 +304,25 @@ fn skip_struct(
     } else {
         type_info.as_ref().unwrap()
     };
-    let field_infos = type_info_value.get_type_meta().get_field_infos().to_vec();
+    let type_meta = type_info_value.get_type_meta();
+    if ENABLE_FORY_DEBUG_OUTPUT {
+        eprintln!(
+            "[skip_struct] type_name: {:?}, num_fields: {}",
+            type_meta.get_type_name(),
+            type_meta.get_field_infos().len()
+        );
+    }
+    let field_infos = type_meta.get_field_infos().to_vec();
     context.inc_depth()?;
     for field_info in field_infos.iter() {
+        if ENABLE_FORY_DEBUG_OUTPUT {
+            eprintln!(
+                "[skip_struct] field: {:?}, type_id: {}, internal_id: {}",
+                field_info.field_name,
+                field_info.field_type.type_id,
+                field_info.field_type.type_id & 0xff
+            );
+        }
         let read_ref_flag = util::field_need_write_ref_into(
             field_info.field_type.type_id,
             field_info.field_type.nullable,
@@ -328,23 +408,42 @@ fn skip_value(
         if ref_flag == (RefFlag::Null as i8) {
             return Ok(());
         }
+        if ref_flag == (RefFlag::Ref as i8) {
+            // Reference to already-seen object, skip the reference index
+            let _ref_index = context.reader.read_varuint32()?;
+            return Ok(());
+        }
+        // RefValue (0) or NotNullValue (-1) means we need to read the actual object
     }
     let type_id_num = field_type.type_id;
 
     // Check if it's a user-defined type (high bits set, meaning type_id > 255)
     if type_id_num > 255 {
         let internal_id = type_id_num & 0xff;
-        if internal_id == types::COMPATIBLE_STRUCT {
+        // Handle struct-like types including UNKNOWN (0) which is used for polymorphic types
+        if internal_id == types::COMPATIBLE_STRUCT
+            || internal_id == types::STRUCT
+            || internal_id == types::NAMED_STRUCT
+            || internal_id == types::NAMED_COMPATIBLE_STRUCT
+            || internal_id == types::UNKNOWN
+        {
+            // If type_info is provided (from skip_any_value), use skip_struct directly
+            // which won't try to re-read type_id/meta_index. Otherwise use skip_user_struct.
+            if type_info.is_some() {
+                return skip_struct(context, type_id_num, type_info);
+            }
             return skip_user_struct(context, type_id_num);
-        } else if internal_id == types::ENUM {
+        } else if internal_id == types::ENUM || internal_id == types::NAMED_ENUM {
             let _ordinal = context.reader.read_varuint32()?;
             return Ok(());
-        } else if internal_id == types::EXT {
+        } else if internal_id == types::EXT || internal_id == types::NAMED_EXT {
             return skip_user_ext(context, type_id_num);
         } else {
             return Err(Error::type_error(format!(
-                "Unknown type id: {}",
-                type_id_num
+                "Unknown type id: {} (internal_id: {}, type_info provided: {})",
+                type_id_num,
+                internal_id,
+                type_info.is_some()
             )));
         }
     }
@@ -462,6 +561,7 @@ fn skip_value(
             return skip_ext(context, type_id_num, type_info);
         }
         types::UNKNOWN => {
+            // UNKNOWN (0) is used for polymorphic types in cross-language serialization
             return skip_any_value(context, false);
         }
 
