@@ -260,7 +260,7 @@ fn test_string_serializer() {
     let fory = Fory::default().compatible(true).xlang(true);
     let mut buf = Vec::new();
     for s in &test_strings {
-        fory.serialize_to(s, &mut buf).unwrap();
+        fory.serialize_to(&mut buf, s).unwrap();
     }
     fs::write(&data_file_path, buf).unwrap();
 }
@@ -320,34 +320,34 @@ fn test_cross_language_serializer() {
     assert_de!(fory, reader, Color, color);
 
     let mut buf = Vec::new();
-    fory.serialize_to(&true, &mut buf).unwrap();
-    fory.serialize_to(&false, &mut buf).unwrap();
-    fory.serialize_to(&-1, &mut buf).unwrap();
-    fory.serialize_to(&i8::MAX, &mut buf).unwrap();
-    fory.serialize_to(&i8::MIN, &mut buf).unwrap();
-    fory.serialize_to(&i16::MAX, &mut buf).unwrap();
-    fory.serialize_to(&i16::MIN, &mut buf).unwrap();
-    fory.serialize_to(&i32::MAX, &mut buf).unwrap();
-    fory.serialize_to(&i32::MIN, &mut buf).unwrap();
-    fory.serialize_to(&i64::MAX, &mut buf).unwrap();
-    fory.serialize_to(&i64::MIN, &mut buf).unwrap();
-    fory.serialize_to(&-1f32, &mut buf).unwrap();
-    fory.serialize_to(&-1f64, &mut buf).unwrap();
-    fory.serialize_to(&"str".to_string(), &mut buf).unwrap();
-    fory.serialize_to(&day, &mut buf).unwrap();
-    fory.serialize_to(&instant, &mut buf).unwrap();
-    fory.serialize_to(&vec![true, false], &mut buf).unwrap();
-    fory.serialize_to(&vec![1, i8::MAX as u8], &mut buf)
+    fory.serialize_to(&mut buf, &true).unwrap();
+    fory.serialize_to(&mut buf, &false).unwrap();
+    fory.serialize_to(&mut buf, &-1).unwrap();
+    fory.serialize_to(&mut buf, &i8::MAX).unwrap();
+    fory.serialize_to(&mut buf, &i8::MIN).unwrap();
+    fory.serialize_to(&mut buf, &i16::MAX).unwrap();
+    fory.serialize_to(&mut buf, &i16::MIN).unwrap();
+    fory.serialize_to(&mut buf, &i32::MAX).unwrap();
+    fory.serialize_to(&mut buf, &i32::MIN).unwrap();
+    fory.serialize_to(&mut buf, &i64::MAX).unwrap();
+    fory.serialize_to(&mut buf, &i64::MIN).unwrap();
+    fory.serialize_to(&mut buf, &-1f32).unwrap();
+    fory.serialize_to(&mut buf, &-1f64).unwrap();
+    fory.serialize_to(&mut buf, &"str".to_string()).unwrap();
+    fory.serialize_to(&mut buf, &day).unwrap();
+    fory.serialize_to(&mut buf, &instant).unwrap();
+    fory.serialize_to(&mut buf, &vec![true, false]).unwrap();
+    fory.serialize_to(&mut buf, &vec![1, i8::MAX as u8])
         .unwrap();
-    fory.serialize_to(&vec![1, i16::MAX], &mut buf).unwrap();
-    fory.serialize_to(&vec![1, i32::MAX], &mut buf).unwrap();
-    fory.serialize_to(&vec![1, i64::MAX], &mut buf).unwrap();
-    fory.serialize_to(&vec![1f32, 2f32], &mut buf).unwrap();
-    fory.serialize_to(&vec![1f64, 2f64], &mut buf).unwrap();
-    fory.serialize_to(&str_list, &mut buf).unwrap();
-    fory.serialize_to(&str_set, &mut buf).unwrap();
-    fory.serialize_to(&str_map, &mut buf).unwrap();
-    fory.serialize_to(&color, &mut buf).unwrap();
+    fory.serialize_to(&mut buf, &vec![1, i16::MAX]).unwrap();
+    fory.serialize_to(&mut buf, &vec![1, i32::MAX]).unwrap();
+    fory.serialize_to(&mut buf, &vec![1, i64::MAX]).unwrap();
+    fory.serialize_to(&mut buf, &vec![1f32, 2f32]).unwrap();
+    fory.serialize_to(&mut buf, &vec![1f64, 2f64]).unwrap();
+    fory.serialize_to(&mut buf, &str_list).unwrap();
+    fory.serialize_to(&mut buf, &str_set).unwrap();
+    fory.serialize_to(&mut buf, &str_map).unwrap();
+    fory.serialize_to(&mut buf, &color).unwrap();
     fs::write(&data_file_path, buf).unwrap();
 }
 
@@ -449,10 +449,10 @@ fn test_list() {
     assert_eq!(remote_item_list2, item_list2);
 
     let mut buf = Vec::new();
-    fory.serialize_to(&remote_str_list, &mut buf).unwrap();
-    fory.serialize_to(&remote_str_list2, &mut buf).unwrap();
-    fory.serialize_to(&remote_item_list, &mut buf).unwrap();
-    fory.serialize_to(&remote_item_list2, &mut buf).unwrap();
+    fory.serialize_to(&mut buf, &remote_str_list).unwrap();
+    fory.serialize_to(&mut buf, &remote_str_list2).unwrap();
+    fory.serialize_to(&mut buf, &remote_item_list).unwrap();
+    fory.serialize_to(&mut buf, &remote_item_list2).unwrap();
 
     fs::write(&data_file_path, buf).unwrap();
 }
@@ -565,13 +565,13 @@ fn test_integer() {
     assert_eq!(remote_f6, f6);
 
     let mut buf = Vec::new();
-    fory.serialize_to(&remote_item2, &mut buf).unwrap();
-    fory.serialize_to(&remote_f1, &mut buf).unwrap();
-    fory.serialize_to(&remote_f2, &mut buf).unwrap();
-    fory.serialize_to(&remote_f3, &mut buf).unwrap();
-    fory.serialize_to(&remote_f4, &mut buf).unwrap();
-    fory.serialize_to(&remote_f5, &mut buf).unwrap();
-    fory.serialize_to(&remote_f6, &mut buf).unwrap();
+    fory.serialize_to(&mut buf, &remote_item2).unwrap();
+    fory.serialize_to(&mut buf, &remote_f1).unwrap();
+    fory.serialize_to(&mut buf, &remote_f2).unwrap();
+    fory.serialize_to(&mut buf, &remote_f3).unwrap();
+    fory.serialize_to(&mut buf, &remote_f4).unwrap();
+    fory.serialize_to(&mut buf, &remote_f5).unwrap();
+    fory.serialize_to(&mut buf, &remote_f6).unwrap();
     fs::write(&data_file_path, buf).unwrap();
 }
 
@@ -695,13 +695,13 @@ fn test_consistent_named() {
     }
     let mut buf = Vec::new();
     for _ in 0..3 {
-        fory.serialize_to(&color, &mut buf).unwrap();
+        fory.serialize_to(&mut buf, &color).unwrap();
     }
     for _ in 0..3 {
-        fory.serialize_to(&my_struct, &mut buf).unwrap();
+        fory.serialize_to(&mut buf, &my_struct).unwrap();
     }
     for _ in 0..3 {
-        fory.serialize_to(&my_ext, &mut buf).unwrap();
+        fory.serialize_to(&mut buf, &my_ext).unwrap();
     }
     fs::write(&data_file_path, buf).unwrap();
 }
@@ -714,6 +714,16 @@ struct VersionCheckStruct {
     f3: f64,
 }
 
+#[derive(ForyObject, Debug, PartialEq)]
+struct StructWithList {
+    items: Vec<Option<String>>,
+}
+
+#[derive(ForyObject, Debug, PartialEq)]
+struct StructWithMap {
+    data: HashMap<Option<String>, Option<String>>,
+}
+
 #[test]
 #[ignore]
 fn test_struct_version_check() {
@@ -737,3 +747,267 @@ fn test_struct_version_check() {
     assert_eq!(new_local_obj, local_obj);
     fs::write(&data_file_path, new_bytes).unwrap();
 }
+
+#[test]
+#[ignore]
+fn test_item() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<Item>(102).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    let item1 = Item {
+        name: Some("test_item_1".to_string()),
+    };
+    let item2 = Item {
+        name: Some("test_item_2".to_string()),
+    };
+    let item3 = Item { name: None };
+
+    let remote_item1: Item = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_item1, item1);
+    let remote_item2: Item = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_item2, item2);
+    let remote_item3: Item = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_item3, item3);
+
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &remote_item1).unwrap();
+    fory.serialize_to(&mut buf, &remote_item2).unwrap();
+    fory.serialize_to(&mut buf, &remote_item3).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
+
+#[test]
+#[ignore]
+fn test_color() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<Color>(101).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    let remote_green: Color = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_green, Color::Green);
+    let remote_red: Color = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_red, Color::Red);
+    let remote_blue: Color = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_blue, Color::Blue);
+    let remote_white: Color = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_white, Color::White);
+
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &Color::Green).unwrap();
+    fory.serialize_to(&mut buf, &Color::Red).unwrap();
+    fory.serialize_to(&mut buf, &Color::Blue).unwrap();
+    fory.serialize_to(&mut buf, &Color::White).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
+
+#[test]
+#[ignore]
+fn test_struct_with_list() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<StructWithList>(201).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    let struct1 = StructWithList {
+        items: vec![
+            Some("a".to_string()),
+            Some("b".to_string()),
+            Some("c".to_string()),
+        ],
+    };
+    let struct2 = StructWithList {
+        items: vec![Some("x".to_string()), None, Some("z".to_string())],
+    };
+
+    let remote_struct1: StructWithList = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_struct1, struct1);
+    let remote_struct2: StructWithList = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_struct2, struct2);
+
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &remote_struct1).unwrap();
+    fory.serialize_to(&mut buf, &remote_struct2).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
+
+#[test]
+#[ignore]
+fn test_struct_with_map() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<StructWithMap>(202).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    let struct1 = StructWithMap {
+        data: HashMap::from([
+            (Some("key1".to_string()), Some("value1".to_string())),
+            (Some("key2".to_string()), Some("value2".to_string())),
+        ]),
+    };
+    let struct2 = StructWithMap {
+        data: HashMap::from([
+            (Some("k1".to_string()), None),
+            (None, Some("v2".to_string())),
+        ]),
+    };
+
+    let remote_struct1: StructWithMap = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_struct1, struct1);
+    let remote_struct2: StructWithMap = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(remote_struct2, struct2);
+
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &remote_struct1).unwrap();
+    fory.serialize_to(&mut buf, &remote_struct2).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
+
+// ============================================================================
+// Polymorphic Container Test Types - Using Box<dyn Any> for runtime polymorphism
+// ============================================================================
+
+use std::any::Any;
+
+#[derive(ForyObject, Debug, PartialEq, Clone)]
+struct Dog {
+    age: i32,
+    name: Option<String>,
+}
+
+#[derive(ForyObject, Debug, PartialEq, Clone)]
+struct Cat {
+    age: i32,
+    lives: i32,
+}
+
+#[derive(ForyObject, Debug)]
+struct AnimalListHolder {
+    animals: Vec<Box<dyn Any>>,
+}
+
+#[derive(ForyObject, Debug)]
+struct AnimalMapHolder {
+    animal_map: HashMap<Option<String>, Box<dyn Any>>,
+}
+
+#[test]
+#[ignore]
+fn test_polymorphic_list() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<Dog>(302).unwrap();
+    fory.register::<Cat>(303).unwrap();
+    fory.register::<AnimalListHolder>(304).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    // Part 1: Read List<Animal> with polymorphic elements (Dog, Cat)
+    let animals: Vec<Box<dyn Any>> = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(animals.len(), 2);
+
+    // First element should be Dog
+    let dog = animals[0]
+        .downcast_ref::<Dog>()
+        .expect("First element should be Dog");
+    assert_eq!(dog.age, 3);
+    assert_eq!(dog.name, Some("Buddy".to_string()));
+
+    // Second element should be Cat
+    let cat = animals[1]
+        .downcast_ref::<Cat>()
+        .expect("Second element should be Cat");
+    assert_eq!(cat.age, 5);
+    assert_eq!(cat.lives, 9);
+
+    // Part 2: Read AnimalListHolder (List<Animal> as struct field)
+    let holder: AnimalListHolder = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(holder.animals.len(), 2);
+
+    let dog2 = holder.animals[0]
+        .downcast_ref::<Dog>()
+        .expect("First holder element should be Dog");
+    assert_eq!(dog2.name, Some("Rex".to_string()));
+
+    let cat2 = holder.animals[1]
+        .downcast_ref::<Cat>()
+        .expect("Second holder element should be Cat");
+    assert_eq!(cat2.lives, 7);
+
+    // Write back
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &animals).unwrap();
+    fory.serialize_to(&mut buf, &holder).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
+
+#[test]
+#[ignore]
+fn test_polymorphic_map() {
+    let data_file_path = get_data_file();
+    let bytes = fs::read(&data_file_path).unwrap();
+
+    let mut fory = Fory::default().compatible(true).xlang(true);
+    fory.register::<Dog>(302).unwrap();
+    fory.register::<Cat>(303).unwrap();
+    fory.register::<AnimalMapHolder>(305).unwrap();
+    let mut reader = Reader::new(bytes.as_slice());
+
+    // Part 1: Read Map<String, Animal> with polymorphic values
+    let animal_map: HashMap<Option<String>, Box<dyn Any>> =
+        fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(animal_map.len(), 2);
+
+    let dog1 = animal_map
+        .get(&Some("dog1".to_string()))
+        .expect("dog1 should exist")
+        .downcast_ref::<Dog>()
+        .expect("dog1 should be Dog");
+    assert_eq!(dog1.name, Some("Rex".to_string()));
+    assert_eq!(dog1.age, 2);
+
+    let cat1 = animal_map
+        .get(&Some("cat1".to_string()))
+        .expect("cat1 should exist")
+        .downcast_ref::<Cat>()
+        .expect("cat1 should be Cat");
+    assert_eq!(cat1.lives, 9);
+    assert_eq!(cat1.age, 4);
+
+    // Part 2: Read AnimalMapHolder (Map<String, Animal> as struct field)
+    let holder: AnimalMapHolder = fory.deserialize_from(&mut reader).unwrap();
+    assert_eq!(holder.animal_map.len(), 2);
+
+    let my_dog = holder
+        .animal_map
+        .get(&Some("myDog".to_string()))
+        .expect("myDog should exist")
+        .downcast_ref::<Dog>()
+        .expect("myDog should be Dog");
+    assert_eq!(my_dog.name, Some("Fido".to_string()));
+
+    let my_cat = holder
+        .animal_map
+        .get(&Some("myCat".to_string()))
+        .expect("myCat should exist")
+        .downcast_ref::<Cat>()
+        .expect("myCat should be Cat");
+    assert_eq!(my_cat.lives, 8);
+
+    // Write back
+    let mut buf = Vec::new();
+    fory.serialize_to(&mut buf, &animal_map).unwrap();
+    fory.serialize_to(&mut buf, &holder).unwrap();
+    fs::write(&data_file_path, buf).unwrap();
+}
